@@ -15,23 +15,18 @@ data = dataset[0]
 
 train_idx = data.train_mask.nonzero(as_tuple=False).view(-1)
 
-#############################################
-# Original PyG's Code
-############################################
-#train_loader = NeighborSampler(data.edge_index, node_idx=data.train_mask,
+################################
+# Step 1: Using Quiver's sampler
+################################
+# train_loader = NeighborSampler(data.edge_index, node_idx=data.train_mask,
 #                               sizes=[25, 10], batch_size=1024, shuffle=True,
-#                               num_workers=12)
-
-#############################################
-# Integrate Quiver: Using Quiver's sampler
-############################################
+#                               num_workers=12) # Original PyG Code
 train_loader = torch.utils.data.DataLoader(train_idx,
                                            batch_size=1024,
                                            shuffle=True,
-                                           drop_last=True)
-
-csr_topo = quiver.CSRTopo(data.edge_index)
-quiver_sampler = quiver.pyg.GraphSageSampler(csr_topo, sizes=[25, 10], device=0)
+                                           drop_last=True) # Quiver
+csr_topo = quiver.CSRTopo(data.edge_index) # Quiver
+quiver_sampler = quiver.pyg.GraphSageSampler(csr_topo, sizes=[25, 10], device=0, mode='GPU') # Quiver
 
 
 subgraph_loader = NeighborSampler(data.edge_index, node_idx=None, sizes=[-1],
@@ -96,16 +91,13 @@ model = SAGE(dataset.num_features, 256, dataset.num_classes)
 model = model.to(device)
 optimizer = torch.optim.Adam(model.parameters(), lr=0.01)
 
-######################
-# Original Pyg's Code
-######################
-#x = data.x.to(device)
 
-#############################################
-# Integrate Quiver: Using Quiver's Feature
-############################################
-x = quiver.Feature(rank=0, device_list=[0], device_cache_size="110M", cache_policy="device_replicate", csr_topo=csr_topo)
-x.from_cpu_tensor(data.x)
+################################
+# Step 2: Using Quiver's Feature
+################################
+# x = data.x.to(device) # Original PyG Code
+x = quiver.Feature(rank=0, device_list=[0], device_cache_size="4G", cache_policy="device_replicate", csr_topo=csr_topo) # Quiver
+x.from_cpu_tensor(data.x) # Quiver
 
 y = data.y.squeeze().to(device)
 
@@ -117,7 +109,12 @@ def train(epoch):
     pbar.set_description(f'Epoch {epoch:02d}')
 
     total_loss = total_correct = 0
-    for batch_size, n_id, adjs in train_loader:
+    ############################################
+    # Step 3: Training the PyG Model with Quiver
+    ############################################
+    # for batch_size, n_id, adjs in train_loader: # Original PyG Code
+    for seeds in train_loader: # Quiver
+        n_id, batch_size, adjs = quiver_sampler.sample(seeds) # Quiver
         # `adjs` holds a list of `(edge_index, e_id, size)` tuples.
         adjs = [adj.to(device) for adj in adjs]
 
