@@ -38,7 +38,10 @@ def color_mat(access_book, device_list):
     clique2device = {}
     clique_res = []
     all_nodes = list(range(len(device_list)))
-    find_cliques(access_book, clique_res, all_nodes, [], [])
+    if len(device_list) == 8:
+        clique_res = [[0,1,2,3],[4,5,6,7]]
+    else:
+        find_cliques(access_book, clique_res, all_nodes, [], [])
     for index, clique in enumerate(clique_res):
         clique2device[index] = []
         for device_idx in clique:
@@ -207,7 +210,21 @@ class CSRTopo:
         Returns:
             int: edge count
         """
-        return self.indices_.shape[0] - 1
+        return self.indices_.shape[0]
+
+    
+    def share_memory_(self):
+        """
+        Place this CSRTopo in shared memory
+        """
+        self.indptr_.share_memory_()
+        self.indices_.share_memory_()
+        if self.eid_ is not None:
+            self.eid_.share_memory_()
+        
+        if self.feature_order_ is not None:
+            self.feature_order_.share_memory_()
+
 
 
 def reindex_by_config(adj_csr: CSRTopo, graph_feature, gpu_portion):
@@ -238,3 +255,27 @@ def init_p2p(device_list: List[int]):
         device_list (List[int]): device list
     """
     torch_qv.init_p2p(device_list)
+
+
+UNITS = {
+    #
+    "KB": 2**10,
+    "MB": 2**20,
+    "GB": 2**30,
+    #
+    "K": 2**10,
+    "M": 2**20,
+    "G": 2**30,
+}
+
+
+def parse_size(sz) -> int:
+    if isinstance(sz, int):
+        return sz
+    elif isinstance(sz, float):
+        return int(sz)
+    elif isinstance(sz, str):
+        for suf, u in sorted(UNITS.items()):
+            if sz.upper().endswith(suf):
+                return int(float(sz[:-len(suf)]) * u)
+    raise Exception("invalid size: {}".format(sz))
\ No newline at end of file
